<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

use Invertus\ConnectPay\Adapter\Tools as ConnectPayAdapterTools;
use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Adapter\Configuration;
use Invertus\ConnectPay\Exception\ConnectPayApiException;
use Invertus\ConnectPay\Payment\API\InitializePaymentApiRequest;
use Invertus\ConnectPay\Payment\Service\CallBackLinkService;
use Invertus\ConnectPay\Controller\AbstractFrontController;
use Invertus\ConnectPay\OAuth\Validator\PrivateKeyValidator;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ConnectPayCreatePaymentModuleFrontController extends AbstractFrontController
{
    private const FILE_NAME = 'createPayment';

    /** @var ConnectPay */
    public $module;

    public function postProcess()
    {
        /** @var Configuration $configuration */
        $configuration = $this->module->getService(Configuration::class);
        /** @var ConnectPayAdapterTools $tools */
        $tools = $this->module->getService(ConnectPayAdapterTools::class);

        $paymentMethod = $tools->getValue('connectpayPaymentMethod');
        $providerCode = ($tools->getValue('connectpayProvider')) ?
            $tools->getValue('connectpayProvider') : '';

        if ($providerCode !== 'by_card' && !$this->validateBankPaymentCurrency()) {
            $this->errors[] = $this->module->l('For bank transfers only EUR currency is available', self::FILE_NAME);
            $this->redirectWithNotifications($this->getOrderStepPageRedirectLink(3));
        }

        if (!$this->validateSelectedValue($tools->getValue('defaultBankTransfer'), $providerCode)) {
            $this->errors[] = $this->module->l('Please select payment method', self::FILE_NAME);
            $this->redirectWithNotifications($this->getOrderStepPageRedirectLink(3));
        }

        $cart = $this->context->cart;
        $redirectLink = $this->getOrderStepPageRedirectLink(1);

        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            $tools->redirect($redirectLink);
        }

        $errors = $this->userDataValidation($customer);
        if (!empty($errors)) {
            foreach ($errors as $error) {
                $this->errors[] = $error;
            }

            $this->redirectWithNotifications($redirectLink);
        }

        if ($cart->id_customer == 0
            || $cart->id_address_delivery == 0
            || $cart->id_address_invoice == 0
            || !$this->module->active
        ) {
            $tools->redirect($redirectLink);
        }

        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] === $this->module->name) {
                $authorized = true;
                break;
            }
        }

        if (!$authorized) {
            $this->errors[] =
                $this->module->l('This payment method is not available.', self::FILE_NAME);
            $this->redirectWithNotifications($redirectLink);
        }

        // Validate Private Key
        /** @var PrivateKeyValidator $privateKeyValidator */
        $privateKeyValidator = $this->module->getService(PrivateKeyValidator::class);

        if (!$privateKeyValidator->isValid()) {
            $this->errors[] =
                $this->module->l('This payment method is not available.', self::FILE_NAME);
            $this->redirectWithNotifications($redirectLink);
        }

        $currency = $this->context->currency;
        $total = $cart->getOrderTotal();

        $orderId = Order::getOrderByCartId($cart->id);
        if (!$orderId) {
            $this->module->validateOrder(
                $cart->id,
                $configuration->get(
                    ConnectPayConfig::ORDER_STATUS_AWAITING_PAYMENT,
                    (int) $cart->id_shop
                ),
                (float) $total,
                $paymentMethod,
                null,
                [],
                (int) $currency->id,
                false,
                $customer->secure_key
            );
        }

        // InitializeConnectPay
        /** @var InitializePaymentApiRequest $initializeService */
        $initializeService = $this->module->getService(InitializePaymentApiRequest::class);
        try {
            $response = $initializeService->initializePayment($providerCode);
        } catch (ConnectPayApiException $exception) {
            $logs = new ConnectPayLog();
            $logs->message = $exception->getResponse()->getBody()->getContents();
            $logs->payload = $exception->getCode();
            $logs->add();
        }

        if (!$response) {
            /** @var CallBackLinkService $callBackLinkService */
            $callBackLinkService = $this->module->getService(CallBackLinkService::class);
            $tools->redirect($callBackLinkService->getPaymentFailLink());
        }

        $connectPayOrder = new ConnectPayOrder();
        $connectPayOrder->id_order = (int) Order::getOrderByCartId($cart->id);
        $connectPayOrder->id_payment = $response->payment->paymentId;
        $connectPayOrder->add();

        $tools->redirect($response->payment->redirectUrl);
    }

    private function validateSelectedValue($defaultBank, $providerCode) {
        return $defaultBank || (!$defaultBank && !empty($providerCode));
    }

    private function validateBankPaymentCurrency(): bool
    {
        $currency_id = $this->context->cart->id_currency;
        $currency = Currency::getCurrency($currency_id);
        $currency_iso_code = $currency['iso_code'];

        if ($currency_iso_code !== ConnectPayConfig::MANDATORY_PAYMENT_CURRENCY) {
            return false;
        }

        return true;
    }

    private function getOrderStepPageRedirectLink($step) {
        return $this->context->link->getPageLink(
            'order',
            true,
            null,
            [
                'step' => $step,
            ]
        );
    }

    private function userDataValidation($customer)
    {
        $errors = [];
        $firstNameLimit      = 50;
        $lastNameLimit       = 50;
        $emailLimit           = 250;
        $fName = $customer->firstname;
        $lName = $customer->lastname;
        $email = $customer->email;

        if (strlen($fName) > $firstNameLimit) {
            $errors[] = $this->module->l('First name should not be longer than ' . $firstNameLimit . ' characters');
        }

        if (strlen($lName) > $lastNameLimit) {
            $errors[] = $this->module->l('Last name should not be longer than ' . $lastNameLimit . ' characters');
        }

        if (strlen($email) > $emailLimit) {
                $errors[] = $this->module->l('Email should not be longer than ' . $emailLimit . ' characters');
        }

        if (preg_match('/^[\p{L}-]*$/u', $fName) !== 1 || preg_match('/^[\p{L}-]*$/u', $lName) !== 1) {
            $errors[] = $this->module->l('No special symbols allowed');
        }

        return $errors;
    }
}
