<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

use Invertus\ConnectPay\Exception\ConnectPayApiException;
use Invertus\ConnectPay\Infrastructure\Utility\VersionUtility;
use Invertus\ConnectPay\Payment\API\PaymentStatusApiRequest;
use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Payment\Repository\ConnectPayOrderRepository;
use Invertus\ConnectPay\Controller\AbstractFrontController;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ConnectPaySuccessModuleFrontController extends AbstractFrontController
{
    const FILE_NAME = 'success';
    const ORDER_FAIL_PATH = 'fail';
    private const ORDER_PAGE_REDIRECT_DESTINATION = 'order';
    const RESPONSE_CODE_FAILED = 'Failed';
    const RESPONSE_CODE_ABANDONED = 'Abandoned';

    /** @var ConnectPay */
    public $module;

    public function postProcess()
    {
        $cartId = Tools::getValue('cartId');
        $moduleId = Tools::getValue('moduleId');
        $orderId = Tools::getValue('orderId');
        $secureKey = Tools::getValue('secureKey');

        $cart = new Cart($cartId);

        if ($cart->secure_key !== $secureKey) {
            $redirectLink = $this->context->link->getPageLink(
                'order',
                true,
                null,
                [
                    'step' => 1,
                ]
            );

            Tools::redirect($redirectLink);
        }

        // Update order status.
        /** @var PaymentStatusApiRequest $paymentStatusService */
        $paymentStatusService = $this->module->getService(PaymentStatusApiRequest::class);
        try {
            $response = $paymentStatusService->getPaymentDetails((int) $orderId);
            $orderState = $response->status->code;
            $order = new Order($orderId);

            $newState = (isset(ConnectPayConfig::ORDER_STATUSES[$orderState])) ?
                Configuration::get(ConnectPayConfig::ORDER_STATUSES[$orderState]) :
                Configuration::get(ConnectPayConfig::ORDER_STATUS_AWAITING_PAYMENT);

            $order->setCurrentState($newState);

            /** @var ConnectPayOrderRepository $connectPayOrderRepository */
            $connectPayOrderRepository = $this->module->getService(ConnectPayOrderRepository::class);
            /** @var ConnectPayOrder $connectPayOrder */
            $connectPayOrderId = (int) $connectPayOrderRepository->getConnectPayOrderId($orderId);
            $connectPayOrder = new ConnectPayOrder($connectPayOrderId);
            $connectPayOrder->order_finalized = (bool) $response->status->isTerminal;
            $connectPayOrder->save();
        } catch (ConnectPayApiException $exception) {
            $logs = new ConnectPayLog();
            $logs->message = $exception->getResponse()->getBody()->getContents();
            $logs->payload = $exception->getCode();
            $logs->add();
        }

        if (
            !isset($orderState) ||
            $orderState === self::RESPONSE_CODE_FAILED ||
            $orderState === self::RESPONSE_CODE_ABANDONED
        ) {
            $this->redirectCustomerToOrderFailPage($cartId);
        }

        $orderLink = $this->context->link->getPageLink(
            'order-confirmation',
            true,
            null,
            [
                'id_cart' => $cartId,
                'id_module' => $moduleId,
                'id_order' => $orderId,
                'key' => $secureKey,
            ]
        );

        Tools::redirect($orderLink);
    }

    private function redirectCustomerToOrderFailPage($cartId): void
    {
        // For 1.6.
        if (VersionUtility::isPsVersionLowerThan1700()) {
            $this->redirectWithError(
                $this->module->l('We couldn\'t authorize your payment. Please try again.', self::ORDER_FAIL_PATH),
                self::ORDER_PAGE_REDIRECT_DESTINATION
            );
        }

        $this->warning[] = $this->module->l('We couldn\'t authorize your payment. Please try again.', self::ORDER_FAIL_PATH);

        if (Configuration::get(ConnectPayConfig::FAILED_ORDER_CART_SAVE)) {
            $request_params['id_cart'] = $this->cancelOrderAndCreateCart($cartId);
        }

        $request_params['action'] = 'show';

        $this->redirectWithNotifications(
            $this->context->link->getPageLink(
                'cart',
                null,
                $this->context->language->id,
                $request_params,
                false,
                null,
                false
            )
        );
    }

    private function cancelOrderAndCreateCart($cartId) {
        $cart = new Cart((int) $cartId);
        $new_cart = $cart->duplicate();
        $this->context->cookie->id_cart = $new_cart['cart']->id;
        $order_id = Order::getIdByCartId($cartId);
        $order = new Order($order_id);
        $order->setCurrentState((int) Configuration::get('PS_OS_CANCELED'));

        return $new_cart['cart']->id;
    }
}
